Imports System.Collections.ObjectModel

''' <summary>
''' Manages a list with AlarmTrigger objects, and raises an event,
''' when one of the AlarmTrigger objects requires it.
''' </summary>
''' <remarks></remarks>
Public Class AppointmentList
    Inherits Collection(Of AlarmTrigger)
    ''' <summary>
    ''' Is raised when one of the AlarmTrigger objects added to the
    ''' instance fires an Alarm event itself.
    ''' </summary>
    ''' <param name="sender">Reference to the AlarmTrigger object which has raised the event.</param>
    ''' <param name="e">AlarmEventArgs instance which contains parameters to the event.</param>
    ''' <remarks></remarks>
    Public Event Alarm(ByVal sender As Object, ByVal e As AlarmEventArgs)

    'Contains Nothing or the date of the next appointment.
    Private myNextAppointment As Nullable(Of Date)

    'Is triggered, for example, by Add or Insert of the Collection class.
    'Override, because the Alarm event of the object must be wired
    'to the AlarmHandler procedure.
    Protected Overrides Sub InsertItem(ByVal index As Integer, ByVal item As AlarmTrigger)

        'Dynamically wire the Alarm event of the object to the prozedur AlarmHandler.
        AddHandler item.Alarm, AddressOf AlarmHandler

        'Let the basic procedure do what it needs to do
        '(namely put the item at the correct location).
        MyBase.InsertItem(index, item)

        'The list has changedthe next appointment could be different!
        UpdateNextAppointmentProperty()
    End Sub

    'Called via assignment of an item with the Item property.
    'Override, because the Alarm event of the object must be wired
    'to the AlarmHandler procedure.
    Protected Overrides Sub SetItem(ByVal index As Integer, ByVal item As AlarmTrigger)

        'Remove the old item from this location:
        RemoveHandler Me(index).Alarm, AddressOf AlarmHandler

        'Wire the new item
        AddHandler item.Alarm, AddressOf AlarmHandler

        'Let the basic procedure do what it needs to do
        '(namely put the item at the correct location).
        MyBase.SetItem(index, item)
    End Sub

    'Is triggered, for example, by Remove or RemoveAt of the Collection class.
    'Override, to remove the wired event with AlarmHandler.
    Protected Overrides Sub RemoveItem(ByVal index As Integer)

        'Dynamically remove the Alarm event of the object from the procedure AlarmHandler.
        RemoveHandler Me(index).Alarm, AddressOf AlarmHandler

        'Let the basic procedure do what it needs to do
        '(namely remove the item from the list).
        MyBase.RemoveItem(index)

        'The list has changedthe next appointment could be different!
        UpdateNextAppointmentProperty()
    End Sub

    'When all items are cleared the events of all objects are cleared.
    Protected Overrides Sub ClearItems()

        'Remove all items.
        For Each locItem As AlarmTrigger In Me
            RemoveHandler locItem.Alarm, AddressOf AlarmHandler
        Next

        'Call basic routine.
        MyBase.ClearItems()

        'There is no next appointment.
        myNextAppointment = Nothing
    End Sub

    ''' <summary>
    ''' Raises an event as soon as this routine is executed as the
    ''' event handler routine of an item in this collection.
    ''' </summary>
    Private Sub AlarmHandler(ByVal sender As Object, ByVal e As AlarmEventArgs)
        RaiseEvent Alarm(sender, e)
    End Sub

    ''' <summary>
    ''' Searches of the next appointment in the item list.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateNextAppointmentProperty()

        'No items present...
        If Me.Count = 0 Then
            '...so there is no next appointment
            myNextAppointment = Nothing
        Else
            'Search all items for the earliest
            myNextAppointment = Me(0).TimeOfAlarm

            For Each locItem As AlarmTrigger In Me
                If locItem.TimeOfAlarm < myNextAppointment.Value Then
                    myNextAppointment = locItem.TimeOfAlarm
                End If
            Next
        End If
    End Sub

    ''' <summary>
    ''' Returns the next appointment or Nothing.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NextAppointment() As Nullable(Of Date)
        Get
            'Only return value. The search for the earliest date
            'was already performed during the list change.
            Return myNextAppointment.Value
        End Get
    End Property
End Class

Public Class AlarmTrigger

    Private WithEvents myTrigger As Timer
    Private myAlarmText As String
    Private myTimeOfAlarm As Date
    Private myIsAlarmActivated As Boolean
    Private myThreashold As Integer = 2

    ''' <summary>
    ''' Raised, if a certain time is reached.
    ''' </summary>
    ''' <param name="Sender">Object, which raised this event.</param>
    ''' <param name="e">AlarmEventArgs containing further information about the event.</param>
    ''' <remarks></remarks>
    Public Event Alarm(ByVal Sender As Object, ByVal e As AlarmEventArgs)

    Sub New(ByVal timeOfAlarm As Date)
        Me.TimeOfAlarm = timeOfAlarm
    End Sub

    Sub New(ByVal timeOfAlarm As Date, reason As String, ByVal activated As Boolean)
        Me.TimeOfAlarm = timeOfAlarm
        Me.AlarmActivated = activated
        Me.Reason = reason
    End Sub

    ''' <summary>
    ''' Sets or gets if the alarm is activated (true) or not (false).
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property AlarmActivated() As Boolean
        Get
            Return myIsAlarmActivated
        End Get
        Set(ByVal value As Boolean)
            If value <> myIsAlarmActivated Then
                If value Then
                    myTrigger = New Timer()
                    myTrigger.Interval = 1000
                    myTrigger.Start()
                    myIsAlarmActivated = True
                Else
                    myTrigger.Dispose()
                End If
            End If

        End Set
    End Property

    ''' <summary>
    ''' Sets or gets at which time the alarm - activation presumed - will go off.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Property TimeOfAlarm() As Date
        Get
            Return myTimeOfAlarm
        End Get
        Set(ByVal value As Date)
            'Prevent the alarm time to be in the past. 
            value = Date.Now.Date.Add(value.TimeOfDay)
            If value < Date.Now Then
                value = value.AddDays(1)
            End If
            myTimeOfAlarm = value
        End Set
    End Property

    Private Sub myTrigger_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles myTrigger.Tick
        If myTimeOfAlarm < DateTime.Now Then
            Dim locAlarmTimeEventArgs As New AlarmEventArgs(TimeOfAlarm)
            OnAlarm(locAlarmTimeEventArgs)
            If locAlarmTimeEventArgs.Reset Then
                TimeOfAlarm = locAlarmTimeEventArgs.TimeOfAlarm
            Else
                AlarmActivated = False
            End If
        End If
    End Sub

    ''' <summary>
    ''' Raises the Alarm Event.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Sub OnAlarm(ByVal e As AlarmEventArgs)
        RaiseEvent Alarm(Me, e)
    End Sub

    Public Property Reason As String

    Public Overrides Function ToString() As String
        Return TimeOfAlarm.ToString("dd.MM.yyyy HH:mm:ss  -  ") & Reason
    End Function

End Class

Public Class AlarmEventArgs
    Inherits EventArgs

    Sub New(ByVal timeOfAlarm As Date)
        timeOfAlarm = timeOfAlarm
        Reset = True
    End Sub

    Sub New(ByVal timeOfAlarm As Date, ByVal reset As Boolean)
        timeOfAlarm = timeOfAlarm
        reset = reset
    End Sub

    Public Property TimeOfAlarm() As Date
    Public Property Reset() As Boolean
    Public Property AlarmText As String
End Class

